%=================================================================
% Christmas Tutorial to 'Numerical Methods for PDEs', WS 2001/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%=================================================================
% Numerical solution of the linear 1D advection equation
% with periodic boundary condition.
%   u_t +  a * u_x  = 0  on  x in [alfa,beta], t in [0,T]
%   u(alfa,t) = u(beta,t)
%
% Three Finite Difference Methods can be tried:
% scheme 1: Upstream
% scheme 2: Lax-Friedrichs
% scheme 3: Lax-Wendroff
%
% with two different initial conditions:
% start 1 : step
% start 2 : peak
%
%  u^(n+1) = P * u^n
%
% Figure(2) shows the components of u with respect to the
% eigenbasis of P. (red: real part green: imaginary part)
%
%
% (to be done: check if eigenvectors are normalized
%  scaling of components)
%
% Press 'Q' to quit.
%=================================================================

%--------------------
%list of parameters
%--------------------
% Default setting
alfa =-1;    % left  interval boundary
beta = 1;    % right interval boundary
a    = 1;    % advection velocity
cn   = 0.5;  % Courant number
M    = 12;   % number of subintervals


%c      % c=a*dt/dx here: c = cn
%N      % number of time-steps to be performed
%n      % number of time-steps
%t      % time
%dt     % time-step
%dx;    % grid-space
%i      % space-step counter
%scheme % defines the numeric scheme to be used
%start  % defines the initial condition
%arrays
%x   = zeros(M,1);     % grid for numerical solution
%x10 = zeros(10*M,1);  % 10 times finer grid for exact solution
%u   = zeros(M,1);     % numerical solution
%ue  = zeros(10*M,1);  % exact solution

%--------------------
%input parameters
%--------------------
 disp('Press 1,2 or 3 to choose the scheme.');
 scheme= input('Choose the scheme: ');
 disp('Press 1 or 2 to select the initial data.');
 start = input('Initial condition: ');
 ans = input('Press 1 for using default parameters,\npress 2 for customizing the parameters: ');
 if(ans==2)
   alfa  = input('left  border alfa = ');
   beta  = input('right border beta = ');
   a     = input('advection speed a = ');
   M     = input('number of subintervals M = ');
   cn    = input('Courant number (<1) = ');
 end;
%--------------------
%initialization
%--------------------
 N   = 0;
 n   = 0; %iteration
 t   = 0; %time
 dx  = (beta - alfa)/M;
 x   = [alfa:dx:beta-dx];
 x10 = [alfa:dx/10:beta-dx/10];
 u   = exact(start,alfa,beta,a,x,0);
 ue  = exact(start,alfa,beta,a,x10,0);
 dt  = cn *dx/a;
 c   = cn;
%---------------------------------------
%construction of the iteration matrix P
%---------------------------------------
P   = zeros(M,M);
%% Upwind %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 if(scheme==1)
   if(a>0) %Upwind scheme for a > 0 (left-sided)
     P(1,M)  = c;
     P(1,1)  = 1-c;
     for i=2:M,
       P(i,i-1) = c;
       P(i,i)   = 1-c;
     end;
   else %Upwind scheme for a < 0 (right-sided)
     for i=1:M-1,
       P(i,i)   = 1+c;
       P(i,i+1) = - c;
     end;
     P(M,M)  = 1+c;
     P(M,1)  = - c;
   end;
%% Lax-Friedrichs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  elseif(scheme==2)
    P(1,M) = 0.5*(1+c);
    P(1,2) = 0.5*(1-c);
    for i=2:M-1,
      P(i,i-1) =0.5*(1+c);
      P(i,i+1) =0.5*(1-c);
    end;
    P(M,M-1) = 0.5*(1+c);
    P(M,1)   = 0.5*(1-c);
%% Lax-Wendroff %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  elseif(scheme==3)
    P(1,M) = 0.5*(c+c^2);
    P(1,1) = 1- c^2;
    P(1,2) = 0.5*(c^2-c);
    for i=2:M-1,
      P(i,i-1) = 0.5*(c+c^2);
      P(i,i)   = 1- c^2;
      P(i,i+1) = 0.5*(c^2-c);
    end;
    P(M,M-1) = 0.5*(c+c^2);
    P(M,M)   = 1- c^2;
    P(M,1)   = 0.5*(c^2-c);
  end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%--------------------
% Time-Loop
%--------------------

[EV,ev] = eig(P);
ev = diag(ev);
X  = inv(EV).';
Q  = inv(P);


while(1)
  figure(1);
  visualize;
  figure(2); clf;
  plot3(real(ev),imag(ev),zeros(1,M),'k*');
  line([-1 1],[0 0],[0 0]);
  line([0 0],[-1 1],[0 0]);
  axis([-1 1 -1 1 -2 2]);
  d = u*X;
  for i=1:M,
    handler = line([real(ev(i)),real(ev(i))],[imag(ev(i)),imag(ev(i))],[0,real(d(i))]);
    set(handler,'color','red');
    handler = line([real(ev(i))+0.01,real(ev(i))+0.01],[imag(ev(i)),imag(ev(i))],[0,imag(d(i))]);
    set(handler,'color','green');
  end; %for

  N = input('number of performing time-steps ','s');
  if(isempty(N)==1) N=1;
  elseif(N=='q') break; end;
  N = numeric(N);
  t = t + N*dt;
  n = t/dt;
  if(N>0)
    u  = (P^N * u')';
  elseif(N<0)
    u  = (Q^(-N) * u')';
  end;
  ue = exact(start,alfa,beta,a,x10,t);
end;