%================================================================================
% Christmas Tutorial to 'Numerical Methods for PDEs', WS 01/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%===============================================================================
%  This program visualizes the spectrum of the evolution matrix
%  for some finite difference schemes (left Upwind,
%  Lax-Friedrichs, Lax-Wendroff), which are used to solve the
%  linear scalar advection equation u_t + a*u_x = 0 with periodic
%  boundary conditons.
%  When the matlab graphic window is activated, one can click
%  with a crosshair (reticle) on the marked spots representing the
%  eigenvalues to watch their associated eigenvectors.
%  If the reticule is not well positioned, there will be in the
%  right upper corner the message 'no hit'; otherwise you get
%  the coordinates of the selected eigenvalue.
%  By typing 'Q' and pressing then the right mouse key the reticle
%  will disappear. With the magnifier one can now change the region
%  of the complex plane to be displayed in order to get a close-up,
%  in case that the distance between the eigenvalues is too small.
%  Activating the matlab main window and pressing the space-key, let
%  the reticle reappear. Press ESC to quit.
%
%  Note, that it is not yet possible to change the 'hitting-resolution'.
%  So it might be difficult to select the eigenvalues with the reticle
%  if their distance is below 0.01.
%
%  The three function-parameter are optional. So, the function can be called
%  providing no parameter at all or  only M    or   M and c   or    M,c and mode.
%  It is not possible to provide only c or mode !
%  M    = is the number of grid nodes defining the dimension of the matrix
%  c    = Courant number c = a*dt/dx
%  mode = string variable defining how the eigenvalues are plotted (color etc.)
%
%  Eigenvalues are plotted in figure(1) (without clf)
%  eigenvectors in figure(2).
%
%================================================================================

function [] = spectrum(M,c,mode)
  clear global UP; global UP;
  clear global LF; global LF;
  clear global LW; global LW;
  clear global CD; global CF;
  clear global EV; global EV;
  clear global ev; global ev;

  %default-setting for the function parameter
  if(nargin<3) mode = 'r*'; end;
  if(nargin<2) c    =  0.7; end;
  if(nargin<1) M    =   10; end;

  UP = zeros(M,M);
  CD = zeros(M,M);
  LF = zeros(M,M);
  LW = zeros(M,M);
  P  = zeros(M,M);

%--------------------------------
% left Upwind iteration matrix
%--------------------------------
  UP(1,M) = c;
  UP(1,1) = 1-c;
  for i=2:M-1,
    UP(i,i-1) = c;
    UP(i,i)   = 1-c;
  end;
  UP(M,M-1) = c;
  UP(M,M)   = 1-c;
%--------------------------------
% Central Difference
%--------------------------------
  CD(1,M) =  0.5*c;
  CD(1,1) =      1;
  CD(1,2) = -0.5*c;
  for i=2:M-1,
    CD(i,i-1) =  0.5*c;
    CD(i,i)   =      1;
    CD(i,i+1) = -0.5*c;
  end;
  CD(M,M-1) =  0.5*c;
  CD(M,M)   =      1;
  CD(M,1)   = -0.5*c;
%--------------------------------
% Lax-Friedrichs iteration matrix
%--------------------------------
  LF(1,M) = 0.5*(1+c);
  LF(1,2) = 0.5*(1-c);
  for i=2:M-1,
    LF(i,i-1) = 0.5*(1+c);
    LF(i,i+1) = 0.5*(1-c);
  end;
  LF(M,M-1) = 0.5*(1+c);
  LF(M,1)   = 0.5*(1-c);
%--------------------------------
% Lax-Wendroff iteration matrix
%--------------------------------
  LW(1,M) = 0.5*(c+c^2);
  LW(1,1) = 1-c^2;
  LW(1,2) = 0.5*(c^2-c);
  for i=2:M-1,
    LW(i,i-1) = 0.5*(c+c^2);
    LW(i,i)   = 1-c^2;
    LW(i,i+1) = 0.5*(c^2-c);
  end;
  LW(M,M-1) = 0.5*(c+c^2);
  LW(M,M)   = 1-c^2;
  LW(M,1) = 0.5*(c^2-c);
%--------------------------------
% visualization of the spectrum
%--------------------------------

ans = input(' Press 1 for UPWIND\n Press 2 for CENTRAL DIFFERENCE\n Press 3 for LAX-FRIEDRICHS\n Press 4 for LAX-WENDROFF\n select: ');
if    (ans == 1)  P = UP;
elseif(ans == 2)  P = CD;
elseif(ans == 3)  P = LF;
elseif(ans == 4)  P = LW;
end;

[EV,ev] = eig(P);
ev = diag(ev);
figure(1);
plot(real(ev),imag(ev),mode);
axis([-1.5 1.5 -1.5 1.5]);
%axis equal;


%--------------------------------
% visualization of eigenvectors
%--------------------------------
button=0;
alfa = -1;
beta =  1;
dx = (beta-alfa)/M;
x = [-1:dx:1-dx];
while(1)   % button ~= 27)
  while(1) % button ~= 113(Q) | button ~= 27)
      hit = 0;
      [X,Y,button] = ginput(1);
      for i=1:length(ev),
        if( abs(real(ev(i))-X) < 0.01 & abs(imag(ev(i))-Y) < 0.01 )
          hit=1;
          break;
        end; %if
      end; %for
      if(hit==0)
        legend(['no hit']);
      else
        legend([num2str(real(ev(i))) blanks(2) num2str(imag(ev(i)))]);
        uev = EV(:,i)';
        figure(2); clf;
        hold on;
        plot([x beta],[real(uev) real(uev(1))],'r-x');
        plot([x beta],[imag(uev) imag(uev(1))],'b-x');
        legend('Re(uev)','Im(uev)');
        axis([alfa beta -1 1]);
        hold off;
        figure(1);
        end; %else
      if(button == 113) break; end;
      if(button ==  27) break; end;
    end; %while
    if(button == 27) break;
    else pause;      end;
  end; %while



